#include <OB/CORBA.h>
#include "Hello_impl.h"

#include <fstream.h>

int run(CORBA::ORB_ptr);

int main(int argc, char* argv[]) {
	int status = EXIT_SUCCESS;
	CORBA::ORB_var orb;

	try {
		orb = CORBA::ORB_init(argc, argv);
		status = run(orb);
	}catch(const CORBA::Exception&){
		status = EXIT_FAILURE; 
	}

	if(!CORBA::is_nil(orb)) {
		try {
			orb -> destroy();
		} catch (const CORBA::Exception&){
			status = EXIT_FAILURE;
		}
	}
	return status;
}

int run(CORBA::ORB_ptr orb) {
	// Resolve Root POA
	CORBA::Object_var poaOBj =
		orb -> resolve_initial_references("RootPOA");
	
	PortableServer::POA_var rootPoa =
		PortableServer::POA::_narrow(poaOBj);
	// Get a Reference to the POA Manager
	PortableServer::POAManager_var manager =
		rootPoa -> the_POAManager();
	// Create implementation object
	Hello_impl* helloImpl = new Hello_impl();
	PortableServer::ServantBase_var servant = helloImpl;
	Hello_var hello = helloImpl -> _this();
	// Save reference
	CORBA::String_var s = orb -> object_to_string(hello);
	const char* refFile = "Server.ref";
	ofstream out(refFile);
	out << s << endl;
	out.close();

	manager -> activate();
	orb -> run();

	return EXIT_SUCCESS;
}
